%SCREXAMPLE 
%  A script to test the Weeks method on F(s) expressions with known f(t):
%
%  A) Bessel function
%  F(s) = '1./sqrt(s.^2+1)'
%  f(t) = 'besselj(0,t)'
%  Timevec = 10:10:60;
%
%  B) A-S_29.3.019
%  F(s) = '1./(s.*(s.*s+9/25))'
%  f(t) = '(25/9)*(1-cos(3t/5))'
%  Timevec = 1:2:11;
%
%  C) A-S_29.3.079
%  F(s) = 'exp(1./s)./(s.^(1.5))'
%  f(t) = 'sinh(2*sqrt(t))/sqrt(pi)'
%  Timevec = 1:2:11;
%
%  D) A-S_29.3.011 (challenging case for numerical inversion, not finite at t=0)
%  F(s) = '(s+0.6).^(-9/11)'
%  f(t) = '(t.^(-2/11))./(gamma(9/11)*exp(0.6*t))'
%  Timevec = 0.5:0.5:3;
%
%  The script implements a few different tests.
%  The first tests demonstrate how to call the core Weeks method function without the wrapper.
%  The additional tests are of the code with the wrapper.
%  A-1) Core Weeks Method: without a search (sigma=1,b=1) 
%  A-2) Core Weeks Method: CPU Local Search  
%  A-3) Core Weeks Method: CPU Global Search 
%  A-4) Core Weeks Method: GPU Global Search
%  A-5) Wrapped Weeks Method: CPU Local Search
%  A-6) Wrapped Weeks Method: GPU Global Search
%  A-7) Clenshaw Algorithm/Wrapped Weeks Method: CPU Local Search
%  B-1) Wrapped Weeks Method: CPU Local Search
%  B-2) Wrapped Weeks Method: GPU Global Search
%  C-1) Wrapped Weeks Method: CPU Local Search
%  C-2) Wrapped Weeks Method: GPU Global Search
%  D-1) Wrapped Weeks Method: CPU Local Search
%  D-2) Wrapped Weeks Method: GPU Global Search
%
%  Finally, this examples script calls a function to verify that the GPU
%  hardware and JACKET software are avilable (wfnCheckGPUsoft).  
%  If it returns a fail value, this script will only compute the CPU examples.
%
%  References:
%  The numbers of the examples are those found in:
%  1) "Inversion of noise-free Laplace transforms: 
%  Towards a standardized set of test problems", P. Valko and S. Vadja
%  Inverse Problems in Engineering, volume 10, no. 5, pp. 467-483 (2002).  
%  www.pe.tamu.edu/valko/public%5Fhtml/Nil/Tests/index.htm
%  2) "Algorithms for Parameter Selection in the Weeks Method for Inverting 
%  the Laplace Transform", A. Weideman, SIAM J. on Scientific Computing, 
%  volume 21, pp. 111-128 (1999). 
%  http://dip.sun.ac.za/~weideman/research/weeks.html
%
%  Author: 
%  Patrick Kano, Moysey Brio - 2011
%
%  Modification Date [M/D/Y]:
%  03/30/2011 - Version 1.0

clear all;
close all;
format compact;
format long g;

%%%%%%%%%%%%%%%

disp('------------------------------------------------------------');
disp('Test A-1');
disp('Core Weeks Method: without a search (sigma=1,b=1)');

Fs = '1./sqrt(s.^2+1)'
ft = 'besselj(0,t)'
Timevec = 10:10:60;
sigmaPinput = ones(1,6);
bPinput = ones(1,6);

fexact = feval(inline(ft,'t'),Timevec);

Nvec = [8 16 32 64 128];

for nidx=1:length(Nvec)
 NLag = Nvec(nidx);
 
 for tidx=1:length(Timevec)
  tic;
  [fapprox(nidx,tidx),sigmaP(nidx,tidx),bP(nidx,tidx)] =...
  wfnWeeksCore(Fs,Timevec(tidx),NLag,sigmaPinput(tidx),bPinput(tidx));
  Runtime(nidx,tidx) = toc;
 end %tidx
 
 MeaAbsError(nidx,:) = 100*abs(fexact-fapprox(nidx,:));    %percent
 MeaRelError(nidx,:) = MeaAbsError(nidx,:)./abs(fexact); %percent
end

disp('Time: ');  		disp(Timevec);
disp('Exact: '); 		disp(fexact); 
disp('Approximate: ');      	disp(real(fapprox)); 
disp('Absolute Error %: '); 	disp(MeaAbsError); 
disp('Relative Error %: '); 	disp(MeaRelError); 
disp('Run Time :');         	disp(Runtime);
disp('sigma: '); 		disp(sigmaP);
disp('b:');      		disp(bP);

figure(1);
title('A-1');
hold on;
plot(Timevec,fexact,'-ko','LineWidth',2);
plot(Timevec,real(fapprox(1,:)),'-rx');
plot(Timevec,real(fapprox(2,:)),'-gx');
plot(Timevec,real(fapprox(3,:)),'-bx');
plot(Timevec,real(fapprox(4,:)),'-cx');
plot(Timevec,real(fapprox(5,:)),'-mx');
hold off;
axis([0 60 -0.5 0.2]);
xlabel('Time [sec]');
legend('Exact','N=8','N=16','N=32','N=64','N=128');
drawnow;

saveas(gcf,[pwd,filesep,'EXAMPLE_FIGURES',filesep,'A_1.jpg']);
clear all;

%%%%%%%%%%%%%%%

disp('------------------------------------------------------------');
disp('Test A-2');
disp('Core Weeks Method: CPU Local Search');

Fs = '1./sqrt(s.^2+1)'
ft = 'besselj(0,t)'
Timevec = 10:10:60;
sigmamin = 0;
sigmamax = 20;
bmax = 20;
SearchSwitch = 0; %CPU-local
tols = 0.2;
tolb = 0.2;

fexact = feval(inline(ft,'t'),Timevec);

Nvec = [8 16 32 64 128];

for nidx=1:length(Nvec)
 NLag = Nvec(nidx);
 
 for tidx=1:length(Timevec)
  tic;
  [fapprox(nidx,tidx),sigmaP(nidx,tidx),bP(nidx,tidx)] =...
  wfnWeeksCore(Fs,Timevec(tidx),NLag,sigmamin,sigmamax,bmax,SearchSwitch,tols,tolb);
  Runtime(nidx,tidx) = toc;
 end %tidx

 MeaAbsError(nidx,:) = 100*abs(fexact-fapprox(nidx,:));    %percent
 MeaRelError(nidx,:) = MeaAbsError(nidx,:)./abs(fexact); %percent
end

disp('Time: ');  		disp(Timevec);
disp('Exact: '); 		disp(fexact); 
disp('Approximate: ');      	disp(real(fapprox)); 
disp('Absolute Error %: '); 	disp(MeaAbsError); 
disp('Relative Error %: '); 	disp(MeaRelError); 
disp('Run Time :');         	disp(Runtime);
disp('sigma: '); 		disp(sigmaP);
disp('b:');      		disp(bP);

figure(2);
title('A-2');
hold on;
plot(Timevec,fexact,'-ko','LineWidth',2);
plot(Timevec,real(fapprox(1,:)),'-rx');
plot(Timevec,real(fapprox(2,:)),'-gx');
plot(Timevec,real(fapprox(3,:)),'-bx');
plot(Timevec,real(fapprox(4,:)),'-cx');
plot(Timevec,real(fapprox(5,:)),'-mx');
hold off;
xlabel('Time [sec]');
legend('Exact','N=8','N=16','N=32','N=64','N=128','Location','southeast');
drawnow;

saveas(gcf,[pwd,filesep,'EXAMPLE_FIGURES',filesep,'A_2.jpg']);
clear all;

%%%%%%%%%%%%%%%

disp('------------------------------------------------------------');
disp('Test A-3');
disp('Core Weeks Method: CPU Global Local (slow)');

Fs = '1./sqrt(s.^2+1)'
ft = 'besselj(0,t)'
Timevec = 10:10:60;
sigmamin = 0;
sigmamax = 20;
bmax = 20;
SearchSwitch = 1; %CPU-global
tols = 0.2;
tolb = 0.2;

fexact = feval(inline(ft,'t'),Timevec);

Nvec = [8 16 32 64 128];

for nidx=1:length(Nvec)
 NLag = Nvec(nidx);
 
 for tidx=1:length(Timevec)
  tic;
  [fapprox(nidx,tidx),sigmaP(nidx,tidx),bP(nidx,tidx)] =...
  wfnWeeksCore(Fs,Timevec(tidx),NLag,sigmamin,sigmamax,bmax,SearchSwitch,tols,tolb);
  Runtime(nidx,tidx) = toc;
 end %tidx

 MeaAbsError(nidx,:) = 100*abs(fexact-fapprox(nidx,:));    %percent
 MeaRelError(nidx,:) = MeaAbsError(nidx,:)./abs(fexact); %percent
end

disp('Time: ');  		disp(Timevec);
disp('Exact: '); 		disp(fexact); 
disp('Approximate: ');      	disp(real(fapprox)); 
disp('Absolute Error %: '); 	disp(MeaAbsError); 
disp('Relative Error %: '); 	disp(MeaRelError); 
disp('Run Time :');         	disp(Runtime);
disp('sigma: '); 		disp(sigmaP);
disp('b:');      		disp(bP);

figure(3);
title('A-3');
hold on;
plot(Timevec,fexact,'-ko','LineWidth',2);
plot(Timevec,real(fapprox(1,:)),'-rx');
plot(Timevec,real(fapprox(2,:)),'-gx');
plot(Timevec,real(fapprox(3,:)),'-bx');
plot(Timevec,real(fapprox(4,:)),'-cx');
plot(Timevec,real(fapprox(5,:)),'-mx');
hold off;
xlabel('Time [sec]');
legend('Exact','N=8','N=16','N=32','N=64','N=128');
drawnow;

saveas(gcf,[pwd,filesep,'EXAMPLE_FIGURES',filesep,'A_3.jpg']);
clear all;

%%%%%%%%%%%%%%%

disp('------------------------------------------------------------');
disp('Test A-4');
disp('Core Weeks Method: GPU Global Search (fast)');

JacketFlag = wfnCheckGPUsoft;
if(JacketFlag==1)

Fs = '1./sqrt(s.^2+1)'
ft = 'besselj(0,t)'
Timevec = 10:10:60;
sigmamin = 0;
sigmamax = 20;
bmax = 20;
SearchSwitch = 2; %GPU-global-JACKET
tols = 0.2;
tolb = 0.2;

fexact = feval(inline(ft,'t'),Timevec);

Nvec = [8 16 32 64 128];

for nidx=1:length(Nvec)
 NLag = Nvec(nidx);
 
 for tidx=1:length(Timevec)
  tic;
  [fapprox(nidx,tidx),sigmaP(nidx,tidx),bP(nidx,tidx)] =...
  wfnWeeksCore(Fs,Timevec(tidx),NLag,sigmamin,sigmamax,bmax,SearchSwitch,tols,tolb);
  Runtime(nidx,tidx) = toc;
 end %tidx

 MeaAbsError(nidx,:) = 100*abs(fexact-fapprox(nidx,:));    %percent
 MeaRelError(nidx,:) = MeaAbsError(nidx,:)./abs(fexact); %percent
end

disp('Time: ');  		disp(Timevec);
disp('Exact: '); 		disp(fexact); 
disp('Approximate: ');      	disp(real(fapprox)); 
disp('Absolute Error %: '); 	disp(MeaAbsError); 
disp('Relative Error %: '); 	disp(MeaRelError); 
disp('Run Time :');         	disp(Runtime);
disp('sigma: '); 		disp(sigmaP);
disp('b:');      		disp(bP);

figure(4);
title('A-4');
hold on;
plot(Timevec,fexact,'-ko','LineWidth',2);
plot(Timevec,real(fapprox(1,:)),'-rx');
plot(Timevec,real(fapprox(2,:)),'-gx');
plot(Timevec,real(fapprox(3,:)),'-bx');
plot(Timevec,real(fapprox(4,:)),'-cx');
plot(Timevec,real(fapprox(5,:)),'-mx');
hold off;
xlabel('Time [sec]');
legend('Exact','N=8','N=16','N=32','N=64','N=128');
drawnow;

saveas(gcf,[pwd,filesep,'EXAMPLE_FIGURES',filesep,'A_4.jpg']);
clear all;

end %JacketFlag

%%%%%%%%%%%%%%%

disp('------------------------------------------------------------');
disp('Test A-5');
disp('Wrapped Weeks Method: CPU Local Search');

Fs = '1./sqrt(s.^2+1)'
ft = 'besselj(0,t)'
Timevec = 10:10:60;
RelErrorTol = [0.01 0.01 0.01 0.01 0.01 0.01]; %1 percent relative error

fexact = feval(inline(ft,'t'),Timevec);

tic;
[fapprox,sigmaP,bP,EstRelError,EstAbsError,~,~,ToleranceFlag] = WeeksMethod(Fs,Timevec,RelErrorTol);
Runtime = toc;

MeaAbsError = 100*abs(fexact-fapprox);    %percent
MeaRelError = MeaAbsError./abs(fexact); %percent

disp('Time: ');  			disp(Timevec);
disp('Exact: '); 			disp(fexact); 
disp('Approximate: ');      		disp(real(fapprox)); 
disp('Measured Absolute Error %: ');  	disp(MeaAbsError); 
disp('Estimated Absolute Error %: '); 	disp(EstAbsError*100);
disp('Measured Relative Error %: ');  	disp(MeaRelError); 
disp('Estimated Relative Error %: '); 	disp(EstRelError*100);
disp('Run Time :');        		disp(Runtime');
disp('sigma: '); 			disp(sigmaP);
disp('b:');      			disp(bP);
disp('Tolerance Flag: ');		disp(ToleranceFlag);

figure(5);
title('A-5');
hold on;
plot(Timevec,fexact,'-ko','LineWidth',2);
plot(Timevec,real(fapprox),'-rx');
hold off;
xlabel('Time [sec]');
legend('Exact','Approximate');
drawnow;

saveas(gcf,[pwd,filesep,'EXAMPLE_FIGURES',filesep,'A_5.jpg']);
clear all;

%%%%%%%%%%%%%%%

disp('------------------------------------------------------------');
disp('Test A-6');
disp('Wrapped Weeks Method: GPU Global Search');

JacketFlag = wfnCheckGPUsoft;
if(JacketFlag==1)

Fs = '1./sqrt(s.^2+1)'
ft = 'besselj(0,t)'
Timevec = 10:10:60;
RelErrorTol = [0.01 0.01 0.01 0.01 0.01 0.01]; %1 percent relative error

fexact = feval(inline(ft,'t'),Timevec);

tic;
[fapprox,sigmaP,bP,EstRelError,EstAbsError,~,~,ToleranceFlag] = WeeksMethod(Fs,Timevec,RelErrorTol,2);
Runtime = toc;

MeaAbsError = 100*abs(fexact-fapprox);    %percent
MeaRelError = MeaAbsError./abs(fexact); %percent

disp('Time: ');  			disp(Timevec);
disp('Exact: '); 			disp(fexact); 
disp('Approximate: ');      		disp(real(fapprox)); 
disp('Measured Absolute Error %: ');  	disp(MeaAbsError); 
disp('Estimated Absolute Error %: '); 	disp(EstAbsError*100);
disp('Measured Relative Error %: ');  	disp(MeaRelError); 
disp('Estimated Relative Error %: '); 	disp(EstRelError*100);
disp('Run Time :');        		disp(Runtime');
disp('sigma: '); 			disp(sigmaP);
disp('b:');      			disp(bP);
disp('Tolerance Flag: ');		disp(ToleranceFlag);

figure(6);
title('A-6');
hold on;
plot(Timevec,fexact,'-ko','LineWidth',2);
plot(Timevec,real(fapprox),'-rx');
hold off;
xlabel('Time [sec]');
legend('Exact','Approximate');
drawnow;

saveas(gcf,[pwd,filesep,'EXAMPLE_FIGURES',filesep,'A_6.jpg']);
clear all;

end %JacketFlag

%%%%%%%%%%%%%%%
disp('------------------------------------------------------------');
disp('Test A-7');
disp('Clenshaw Algorithm/Wrapped Weeks Method: CPU Local Search');

%This purpose of this example is to show that it is in principle possible to 
%use the Laguerre coefficients directly from the inversion at a single time
%to rapidly compute a coarser estimate of f(t) at multiple times.
%An alternative is to iterpolate (sigma,b) from a coarse time sampling to a finer time sampling
%and then evaluate the core Weeks Function at these values.
%The tool returns sufficient information to the user to investigate multiple options.

tic;
Fs = '1./sqrt(s.^2+1)'
ft = 'besselj(0,t)'
Timevec = 60;

[~,sigmaP,bP,~,~,~,~,~,LaguerreCoef] = WeeksMethod(Fs,Timevec);

Timeinterp = linspace(0,60,1000)'; %covert to a column vector

%Extract the vector of coefficients from the cell array
vecLaguerreCoef = LaguerreCoef{1}';
NLag = length(vecLaguerreCoef);

fapprox = wfnClenshaw(NLag,Timeinterp,sigmaP,bP,vecLaguerreCoef);

%Compare with the analytic result
fexact = feval(inline(ft,'t'),Timeinterp);
MeaAbsError = 100*abs(fexact-fapprox);    %percent
MeaRelError = MeaAbsError./abs(fexact); %percent

Runtime = toc

figure(7);
title('A-7: Solution');
hold on;
plot(Timeinterp,fexact,'-ko','LineWidth',2);
plot(Timeinterp,real(fapprox),'-rx');
hold off;
xlabel('Time [sec]');
legend('Exact','Approximate');
drawnow;
saveas(gcf,[pwd,filesep,'EXAMPLE_FIGURES',filesep,'A_7_soln.jpg']);

figure(8);
plot(Timeinterp,log10(MeaRelError),'-ko');
title('A-7: Relative Error');
xlabel('Time [sec]');
ylabel('log10(Relative Error)');
drawnow;
saveas(gcf,[pwd,filesep,'EXAMPLE_FIGURES',filesep,'A_7_error.jpg']);

clear all;

%%%%%%%%%%%%%%%

%B) A-S_29.3.019
disp('------------------------------------------------------------');
disp('Test B-1');
disp('Wrapped Weeks Method: CPU Local Search');

Fs = '1./(s.*(s.*s+9/25))'
ft = '(25/9)*(1-cos(3*t/5))'
Timevec = 1:2:11;
RelErrorTol = [0.01 0.01 0.01 0.01 0.01 0.01]; %1 percent relative error

fexact = feval(inline(ft,'t'),Timevec);

tic;
[fapprox,sigmaP,bP,EstRelError,EstAbsError,~,~,ToleranceFlag] = WeeksMethod(Fs,Timevec,RelErrorTol);
Runtime = toc;

MeaAbsError = 100*abs(fexact-fapprox);    %percent
MeaRelError = MeaAbsError./abs(fexact); %percent

disp('Time: ');  			disp(Timevec);
disp('Exact: '); 			disp(fexact); 
disp('Approximate: ');      		disp(real(fapprox)); 
disp('Measured Absolute Error %: ');  	disp(MeaAbsError); 
disp('Estimated Absolute Error %: '); 	disp(EstAbsError*100);
disp('Measured Relative Error %: ');  	disp(MeaRelError); 
disp('Estimated Relative Error %: '); 	disp(EstRelError*100);
disp('Run Time :');        		disp(Runtime');
disp('sigma: '); 			disp(sigmaP);
disp('b:');      			disp(bP);
disp('Tolerance Flag: ');		disp(ToleranceFlag);

figure(9);
title('B-1');
hold on;
plot(Timevec,fexact,'-ko','LineWidth',2);
plot(Timevec,real(fapprox),'-rx');
hold off;
xlabel('Time [sec]');
legend('Exact','Approximate');
drawnow;

saveas(gcf,[pwd,filesep,'EXAMPLE_FIGURES',filesep,'B_1.jpg']);

clear now;

%%%%%%%%%%%%%%%

%B) A-S_29.3.019
disp('------------------------------------------------------------');
disp('Test B-2');
disp('Wrapped Weeks Method: GPU Global Search');

JacketFlag = wfnCheckGPUsoft;
if(JacketFlag==1)

Fs = '1./(s.*(s.*s+9/25))'
ft = '(25/9)*(1-cos(3*t/5))'
Timevec = 1:2:11;
RelErrorTol = [0.01 0.01 0.01 0.01 0.01 0.01]; %1 percent relative error

fexact = feval(inline(ft,'t'),Timevec);

tic;
[fapprox,sigmaP,bP,EstRelError,EstAbsError,~,~,ToleranceFlag] = WeeksMethod(Fs,Timevec,RelErrorTol,2);
Runtime = toc;

MeaAbsError = 100*abs(fexact-fapprox);    %percent
MeaRelError = MeaAbsError./abs(fexact); %percent

disp('Time: ');  			disp(Timevec);
disp('Exact: '); 			disp(fexact); 
disp('Approximate: ');      		disp(real(fapprox)); 
disp('Measured Absolute Error %: ');  	disp(MeaAbsError); 
disp('Estimated Absolute Error %: '); 	disp(EstAbsError*100);
disp('Measured Relative Error %: ');  	disp(MeaRelError); 
disp('Estimated Relative Error %: '); 	disp(EstRelError*100);
disp('Run Time :');        		disp(Runtime');
disp('sigma: '); 			disp(sigmaP);
disp('b:');      			disp(bP);
disp('Tolerance Flag: ');		disp(ToleranceFlag);

figure(10);
title('B-2');
hold on;
plot(Timevec,fexact,'-ko','LineWidth',2);
plot(Timevec,real(fapprox),'-rx');
hold off;
xlabel('Time [sec]');
legend('Exact','Approximate');
drawnow;

saveas(gcf,[pwd,filesep,'EXAMPLE_FIGURES',filesep,'B_2.jpg']);

clear all;

end %JacketFlag

%%%%%%%%%%%%%%%

% C) A-S_29.3.079
disp('------------------------------------------------------------');
disp('Test C-1');
disp('Wrapped Weeks Method: CPU Local Search');

Fs = 'exp(1./s)./(s.^(1.5))';
ft = 'sinh(2*sqrt(t))/sqrt(pi)'
Timevec = 1:2:11;
RelErrorTol = [0.01 0.01 0.01 0.01 0.01 0.01]; %1 percent relative error

fexact = feval(inline(ft,'t'),Timevec);

tic;
[fapprox,sigmaP,bP,EstRelError,EstAbsError,~,~,ToleranceFlag] = WeeksMethod(Fs,Timevec,RelErrorTol);
Runtime = toc;

MeaAbsError = 100*abs(fexact-fapprox);    %percent
MeaRelError = MeaAbsError./abs(fexact); %percent

disp('Time: ');  			disp(Timevec);
disp('Exact: '); 			disp(fexact); 
disp('Approximate: ');      		disp(real(fapprox)); 
disp('Measured Absolute Error %: ');  	disp(MeaAbsError); 
disp('Estimated Absolute Error %: '); 	disp(EstAbsError*100);
disp('Measured Relative Error %: ');  	disp(MeaRelError); 
disp('Estimated Relative Error %: '); 	disp(EstRelError*100);
disp('Run Time :');        		disp(Runtime');
disp('sigma: '); 			disp(sigmaP);
disp('b:');      			disp(bP);
disp('Tolerance Flag: ');		disp(ToleranceFlag);

figure(11);
title('C-1');
hold on;
plot(Timevec,fexact,'-ko','LineWidth',2);
plot(Timevec,real(fapprox),'-rx');
hold off;
xlabel('Time [sec]');
legend('Exact','Approximate');
drawnow;

saveas(gcf,[pwd,filesep,'EXAMPLE_FIGURES',filesep,'C_1.jpg']);

clear all;

%%%%%%%%%%%%%%%

% C) A-S_29.3.079
disp('------------------------------------------------------------');
disp('Test C-2');
disp('Wrapped Weeks Method: GPU Global Search');

JacketFlag = wfnCheckGPUsoft;
if(JacketFlag==1)

Fs = 'exp(1./s)./(s.^(1.5))';
ft = 'sinh(2*sqrt(t))/sqrt(pi)'
Timevec = 1:2:11;
RelErrorTol = [0.01 0.01 0.01 0.01 0.01 0.01]; %1 percent relative error

fexact = feval(inline(ft,'t'),Timevec);

tic;
[fapprox,sigmaP,bP,EstRelError,EstAbsError,~,~,ToleranceFlag] = WeeksMethod(Fs,Timevec,RelErrorTol,2);
Runtime = toc;

MeaAbsError = 100*abs(fexact-fapprox);    %percent
MeaRelError = MeaAbsError./abs(fexact); %percent

disp('Time: ');  			disp(Timevec);
disp('Exact: '); 			disp(fexact); 
disp('Approximate: ');      		disp(real(fapprox)); 
disp('Measured Absolute Error %: ');  	disp(MeaAbsError); 
disp('Estimated Absolute Error %: '); 	disp(EstAbsError*100);
disp('Measured Relative Error %: ');  	disp(MeaRelError); 
disp('Estimated Relative Error %: '); 	disp(EstRelError*100);
disp('Run Time :');        		disp(Runtime');
disp('sigma: '); 			disp(sigmaP);
disp('b:');      			disp(bP);
disp('Tolerance Flag: ');		disp(ToleranceFlag);

figure(12);
title('C-2');
hold on;
plot(Timevec,fexact,'-ko','LineWidth',2);
plot(Timevec,real(fapprox),'-rx');
hold off;
xlabel('Time [sec]');
legend('Exact','Approximate');
drawnow;

saveas(gcf,[pwd,filesep,'EXAMPLE_FIGURES',filesep,'C_2.jpg']);

clear all;

end %JacketFlag

%%%%%%%%%%%%%%%

% D) A-S_29.3.011 (challenging case for numerical inversion)
disp('------------------------------------------------------------');
disp('Test D-1');
disp('Wrapped Weeks Method: CPU Local Search');

Fs = '(s+0.6).^(-9/11)'
ft = '(t.^(-2/11))./(gamma(9/11)*exp(0.6*t))'
Timevec = 0.5:0.5:3;
RelErrorTol = [0.1 0.1 0.1 0.1 0.1 0.1]; %10 percent relative error for all times

fexact = feval(inline(ft,'t'),Timevec);

tic;
[fapprox,sigmaP,bP,EstRelError,EstAbsError,~,~,ToleranceFlag] = WeeksMethod(Fs,Timevec,RelErrorTol);
Runtime = toc;

MeaAbsError = 100*abs(fexact-fapprox);    %percent
MeaRelError = MeaAbsError./abs(fexact); %percent

disp('Time: ');  			disp(Timevec);
disp('Exact: '); 			disp(fexact); 
disp('Approximate: ');      		disp(real(fapprox)); 
disp('Measured Absolute Error %: ');  	disp(MeaAbsError); 
disp('Estimated Absolute Error %: '); 	disp(EstAbsError*100);
disp('Measured Relative Error %: ');  	disp(MeaRelError); 
disp('Estimated Relative Error %: '); 	disp(EstRelError*100);
disp('Run Time :');        		disp(Runtime');
disp('sigma: '); 			disp(sigmaP);
disp('b:');      			disp(bP);
disp('Tolerance Flag: ');		disp(ToleranceFlag);

figure(13);
title('D-1');
hold on;
plot(Timevec,fexact,'-ko','LineWidth',2);
plot(Timevec,real(fapprox),'-rx');
hold off;
xlabel('Time [sec]');
legend('Exact','Approximate');
drawnow;

saveas(gcf,[pwd,filesep,'EXAMPLE_FIGURES',filesep,'D_1.jpg']);

clear all;

%%%%%%%%%%%%%%%

% D) A-S_29.3.011 
disp('------------------------------------------------------------');
disp('Test D-2');
disp('Wrapped Weeks Method: GPU Global Search');

JacketFlag = wfnCheckGPUsoft;
if(JacketFlag==1)

Fs = '(s+0.6).^(-9/11)'
ft = '(t.^(-2/11))./(gamma(9/11)*exp(0.6*t))'
Timevec = 0.5:0.5:3;
RelErrorTol = [0.1 0.1 0.1 0.1 0.1 0.1]; %10 percent relative error for all times

fexact = feval(inline(ft,'t'),Timevec);

tic;
[fapprox,sigmaP,bP,EstRelError,EstAbsError,~,~,ToleranceFlag] = WeeksMethod(Fs,Timevec,RelErrorTol,2);
Runtime = toc;

MeaAbsError = 100*abs(fexact-fapprox);    %percent
MeaRelError = MeaAbsError./abs(fexact); %percent

disp('Time: ');  			disp(Timevec);
disp('Exact: '); 			disp(fexact); 
disp('Approximate: ');      		disp(real(fapprox)); 
disp('Measured Absolute Error %: ');  	disp(MeaAbsError); 
disp('Estimated Absolute Error %: '); 	disp(EstAbsError*100);
disp('Measured Relative Error %: ');  	disp(MeaRelError); 
disp('Estimated Relative Error %: '); 	disp(EstRelError*100);
disp('Run Time :');        		disp(Runtime');
disp('sigma: '); 			disp(sigmaP);
disp('b:');      			disp(bP);
disp('Tolerance Flag: ');		disp(ToleranceFlag);

figure(14);
title('D-2');
hold on;
plot(Timevec,fexact,'-ko','LineWidth',2);
plot(Timevec,real(fapprox),'-rx');
hold off;
xlabel('Time [sec]');
legend('Exact','Approximate');
drawnow;

saveas(gcf,[pwd,filesep,'EXAMPLE_FIGURES',filesep,'D_2.jpg']);

clear all;

end %JacketFlag

%End of Script
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
