function [sigmaP, bP] = wfnParamEst(FLaplace,TimeInput,NLag,sigmin,sigmax,bmax,SearchSwitch,tols,tolb)
%WFNPARAMEST Local and global searchs for optimal Weeks parameters sigma and b  
%  The function implements a local search based on the nested 1D fminbnd approach from J.A.C. Weideman (method 2)
%  and a global search based on a uniform grid sampling in the (sigma,b) plane.
% 
%  Use:
%  [sigmaP, bP] = wfnParamEst(FLaplace,TimeInput,NLag,sigmin,sigmax,bmax,SearchSwitch,tols,tolb)
%
%  Input:
%  FLaplace = a symbolic expression for the Laplace transform space function F(s)
%  TimeInput = time where f(t) is to be computed
%  NLag = number of terms in the Weeks expansion.
%  sigmin = minimum allowed value of sigma
%  sigmax = maximum allowed value of sigma
%  bmax = maximum allowed value of b
%  SearchSwitch = 0-CPU local fminbnd, 1-CPU global, 2-GPU/JACKET global 
%  tols = tolerance on the s parameter
%  tolb = tolerance on the b parameter
%
%  Output:
%  sigmaP = the optimized sigma parameter
%  bP = the optimized b parameter
%
%  Author: 
%  Patrick Kano, Moysey Brio - 2011
%
%  Modification Date [M/D/Y]:
%  03/30/2011 - Version 1.0

if(SearchSwitch==0) %CPU local fminbnd
 
 sigmaP = fminbnd('wfnNestedError',sigmin,sigmax,[0 tols],FLaplace,TimeInput,NLag,bmax,tolb);
 bP = fminbnd('wfncpuErrorEst',0,bmax,[0 tolb],sigmaP,FLaplace,TimeInput,NLag,0); %ErrorFlag=0->total error

else %global grid

 Nsigma = floor((sigmax-sigmin)/tols);
 Nb = floor(bmax/tolb);
 
 Nprod = Nsigma*Nb; 
 
 sigmatemp = linspace(sigmin,sigmax,Nsigma+1); sigmatemp = sigmatemp(2:Nsigma+1); 
 btemp = linspace(0,bmax,Nb+1); btemp = btemp(2:Nb+1);
  
 [smesh,bmesh] = meshgrid(sigmatemp,btemp);
 
 svec = smesh(:);
 bvec = bmesh(:);
    
 switch SearchSwitch
  case 1 %CPU global

   Errorvec = zeros(1,Nprod);

   for nidx=1:Nprod
    Errorvec(nidx) = wfncpuErrorEst(bvec(nidx),svec(nidx),FLaplace,TimeInput,NLag,0);
   end

   [~, minidx] = min(Errorvec);
   sigmaP = svec(minidx);
   bP = bvec(minidx);
  
  case 2 %GPU/JACKET global
   svec = gdouble(svec);
   bvec = gdouble(bvec);
   TimeInput = gdouble(TimeInput);
   
   Errorvec = gzeros(1,Nprod);

   %GPU for loop here
   gfor nidx=1:Nprod
    Errorvec(nidx) = wfnjacErrorEst(bvec(nidx),svec(nidx),FLaplace,gdouble(TimeInput),NLag,0);
   gend

   [~, minidx] = min(double(Errorvec));
   sigmaP = double(svec(minidx));
   bP = double(bvec(minidx)); 

  otherwise
   error('wfnParamEst:SearchSwitch','An incorrect choice for the search method has been made.');
 end %switch
end %function definition
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
